/***************************************************************************/
/* 		This code is part of WWW grabber called pavuk		   */
/*		Copyright (c) 1997 - 2000 Stefan Ondrejicka		   */
/*		(ondrej@idata.sk)					   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#include <string.h>
#include <stdlib.h>

#include "config.h"
#include "jstrans.h"
#include "tools.h"

#ifdef HAVE_REGEX

void js_transform_free(jt)
js_transform_t *jt;
{
	re_free(jt->re);
	_free(jt->transform);
	_free(jt->tag);
	_free(jt->attrib);
	_free(jt);
}

js_transform_t *js_transform_new(pattern, transform, tag, attrib)
char *pattern;
char *transform;
char *tag;
char *attrib;
{
	js_transform_t *rv;
	re_entry *re;

	if (!pattern || !transform || !tag || !attrib)
		return NULL;

	re = re_make(pattern);

	if (!re)
		return NULL;

	rv = _malloc(sizeof(js_transform_t));
	rv->re = re;
	rv->transform = tl_strdup(transform);
	rv->tag = tl_strdup(tag);
	rv->attrib = tl_strdup(attrib);

	return rv;
}

int js_transform_match_tag(jt, tag)
js_transform_t *jt;
char *tag;
{
	int l;

	if (jt->tag[0] == '*')
		return TRUE;

	l = strcspn(tag+1, " \t\r\n>");

	if (!strncasecmp(tag+1, jt->tag, l) &&
	    (l == strlen(jt->tag)))
		return TRUE;

	return FALSE;
}

char *js_transform_apply(jt, attr, nsub, subs)
js_transform_t *jt;
char *attr;
int nsub;
int *subs;
{
	char *rv;
	int n,l;

	l = strcspn(jt->transform, "$");
	rv = tl_strndup(jt->transform, l);

	do {
		if (jt->transform[l] == '$')
		{
			n = atoi(jt->transform+l+1);
			if (n <= nsub)
			{
				rv = tl_str_nappend(rv, attr + subs[2*n],
					subs[2*n+1] - subs[2*n]);
			}
			l += 1 + strspn(jt->transform+l+1, "0123456789");
		}
		n = strcspn(jt->transform+l, "$");
		rv = tl_str_nappend(rv, jt->transform+l, n);
		l += n;
	} while (jt->transform[l]);

	return rv;
}

#if 0
char *js_transform_apply_old(jt, attr, nsub, subs)
js_transform_t *jt;
char *attr;
int nsub;
int *subs;
{
	char *p;
	char *rv;
	int n,l;

	l = strcspn(jt->transform, "$");
	rv = tl_strndup(jt->transform, l);

	do {
		if (jt->transform[l] == '$')
		{
			n = atoi(jt->transform+l+1);
			if (n <= nsub)
			{
				p = tl_strndup(attr + subs[2*n],
					subs[2*n+1] - subs[2*n]);
				rv = tl_str_concat(rv, p, NULL);
				_free(p);
			}
			l += 1 + strspn(jt->transform+l+1, "0123456789");
		}
		n = strcspn(jt->transform+l, "$");
		p = tl_strndup(jt->transform+l, n);
		rv = tl_str_concat(rv, p, NULL);
		_free(p);
		l += n;
	} while (jt->transform[l]);

	return rv;
}
#endif


#endif
